﻿using MapControlSamples.Common;
using System;
using System.Collections.Generic;
using Windows.ApplicationModel.Resources;
using Windows.Devices.Geolocation;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Maps;
using Windows.UI.Xaml.Navigation;
using MapControlSamples.Models;

namespace MapControlSamples
{
    /// <summary>
    /// A page that displays details for a single item within a group.
    /// </summary>
    public sealed partial class BindingCollectionsPage : Page
    {
        private readonly NavigationHelper navigationHelper;
        private readonly ResourceLoader resourceLoader = ResourceLoader.GetForCurrentView("Resources");

        private List<Team> _teams;

        public BindingCollectionsPage()
        {
            this.InitializeComponent();

            this.navigationHelper = new NavigationHelper(this);
            this.navigationHelper.LoadState += this.NavigationHelper_LoadState;
            this.navigationHelper.SaveState += this.NavigationHelper_SaveState;

            // setup the teams
            _teams = setupTeams();
        }

        private List<Team> setupTeams()
        {
            List<Team> teams = new List<Team>();

            // AFC East
            teams.Add(new Team("New England Patriots", "AFC East", "Gillette Stadium", 42.090958, -71.264379));
            teams.Add(new Team("Buffalo Bills", "AFC East", "Ralph Wilson Stadium", 42.773749, -78.786604));
            teams.Add(new Team("Miami Dolphins", "AFC East", "Sun Life Stadium", 25.958038, -80.238270));
            teams.Add(new Team("New York Jets", "AFC East", "Met Life Stadium", 40.812852, -74.073781));

            // AFC West
            teams.Add(new Team("Denver Broncos", "AFC West", "Sports Authority Field", 39.744053, -105.019791));
            teams.Add(new Team("San Diego Chargers", "AFC West", "Qualcomm Stadium", 32.783917, -117.119476));
            teams.Add(new Team("Kansas City Chiefs", "AFC West", "Arrowhead Stadium", 39.048935, -94.483444));
            teams.Add(new Team("Oakland Raiders", "AFC West", "O.co Coliseum", 37.751616, -122.200224));

            // AFC North
            teams.Add(new Team("Cincinnati Bengals", "AFC North", "Paul Brown Stadium", 39.096413, -84.516583));
            teams.Add(new Team("Baltimore Ravens", "AFC North", "M&T Bank Stadium", 39.277926, -76.622232));
            teams.Add(new Team("Pittsburgh Steelers", "AFC North", "Heinz Field", 40.446761, -80.015160));
            teams.Add(new Team("Cleveland Browns", "AFC North", "FirstEnergy Stadium", 41.506074, -81.699161));

            // AFC South
            teams.Add(new Team("Indianapolis Colts", "AFC South", "Lucas Oil Stadium", 39.760058, -86.163386));
            teams.Add(new Team("Houston Texans", "AFC South", "NRG Stadium", 29.684691, -95.410450));
            teams.Add(new Team("Tennessee Titans", "AFC South", "LP Field", 36.166648, -86.770979));
            teams.Add(new Team("Jacksonville Jaguars", "AFC South", "EverBank Field", 30.324611, -81.636495));

            // NFC East
            teams.Add(new Team("Dallas Cowboys", "NFC East", "AT&T Stadium", 32.747243, -97.093979));
            teams.Add(new Team("Philadelphia Eagles", "NFC East", "Lincoln Financial Field", 39.900670, -75.166956));
            teams.Add(new Team("New York Giants", "NFC East", "Met Life Stadium", 40.813055, -74.074230));
            teams.Add(new Team("Washington Redskins", "NFC East", "FedEx Field", 38.907686, -76.865738));

            // NFC West
            teams.Add(new Team("Arizona Cardinals", "NFC West", "University of Phoenix Stadium", 33.527594, -112.262028));
            teams.Add(new Team("San Francisco 49ers", "NFC West", "Levi's Stadium", 37.403260, -121.969394));
            teams.Add(new Team("Seattle Seahawks", "NFC West", "CenturyLink Field", 47.595141, -122.331060));
            teams.Add(new Team("St. Louis Rams", "NFC West", "Edward Jones Dome", 38.632984, -90.187871));

            // NFC North
            teams.Add(new Team("Detroit Lions", "NFC North", "Ford Field", 42.339978, -83.045078));
            teams.Add(new Team("Green Bay Packers", "NFC North", "Lambeau Field", 44.501352, -88.061606));
            teams.Add(new Team("Chicago Bears", "NFC North", "Soldier Field", 41.862373, -87.616088));
            teams.Add(new Team("Minnesota Vikings", "NFC North", "TFC Bank Stadium", 44.976892, -93.224331));

            // NFC South
            teams.Add(new Team("Carolina Panthers", "NFC South", "Bank of America Stadium", 35.225652, -80.852246));
            teams.Add(new Team("New Orleans Saints", "NFC South", "Mercedes-Benz Superdome", 29.951039, -90.080727));
            teams.Add(new Team("Atlanta Falcons", "NFC South", "Georgia Dome", 33.757712, -84.400352));
            teams.Add(new Team("Tampa Bay Buccaneers", "NFC South", "Raymond James Stadium", 27.975874, -82.502776));

            return teams;
        }

        /// <summary>
        /// Gets the <see cref="NavigationHelper"/> associated with this <see cref="Page"/>.
        /// </summary>
        public NavigationHelper NavigationHelper
        {
            get { return this.navigationHelper; }
        }

        /// <summary>
        /// Populates the page with content passed during navigation. Any saved state is also
        /// provided when recreating a page from a prior session.
        /// </summary>
        /// <param name="sender">
        /// The source of the event; typically <see cref="NavigationHelper"/>.
        /// </param>
        /// <param name="e">Event data that provides both the navigation parameter passed to
        /// <see cref="Frame.Navigate(Type, Object)"/> when this page was initially requested and
        /// a dictionary of state preserved by this page during an earlier
        /// session.  The state will be null the first time a page is visited.</param>
        private void NavigationHelper_LoadState(object sender, LoadStateEventArgs e)
        {
            // TODO: Create an appropriate data model for your problem domain to replace the sample data.
        }

        /// <summary>
        /// Preserves state associated with this page in case the application is suspended or the
        /// page is discarded from the navigation cache.  Values must conform to the serialization
        /// requirements of <see cref="SuspensionManager.SessionState"/>.
        /// </summary>
        /// <param name="sender">The source of the event; typically <see cref="NavigationHelper"/>.</param>
        /// <param name="e">Event data that provides an empty dictionary to be populated with
        /// serializable state.</param>
        private void NavigationHelper_SaveState(object sender, SaveStateEventArgs e)
        {
            // TODO: Save the unique state of the page here.
        }

        #region NavigationHelper registration

        /// <summary>
        /// The methods provided in this section are simply used to allow
        /// NavigationHelper to respond to the page's navigation methods.
        /// <para>
        /// Page specific logic should be placed in event handlers for the  
        /// <see cref="NavigationHelper.LoadState"/>
        /// and <see cref="NavigationHelper.SaveState"/>.
        /// The navigation parameter is available in the LoadState method 
        /// in addition to page state preserved during an earlier session.
        /// </para>
        /// </summary>
        /// <param name="e">Provides data for navigation methods and event
        /// handlers that cannot cancel the navigation request.</param>
        protected async override void OnNavigatedTo(NavigationEventArgs e)
        {
            // Call the navigation helper base function.
            this.navigationHelper.OnNavigatedTo(e);

            // Size the Map Control
            myMapControl.Width = Windows.UI.Xaml.Window.Current.Bounds.Width;
            myMapControl.Height = Windows.UI.Xaml.Window.Current.Bounds.Height-120;

            // Set the ItemsSource of the MapItemsControl.
            MapItems.ItemsSource = _teams;

            // create the view bounds
            List<BasicGeoposition> positions = new List<BasicGeoposition>();
            foreach (Team team in _teams)
            {
                positions.Add(team.StadiumLocation.Position);
            }

            // Get the geobounding box
            GeoboundingBox box = GeoboundingBox.TryCompute(positions);
            
            myMapControl.Center = new Geopoint(box.Center);

            // For some reason this function always returns false.
            bool success = await myMapControl.TrySetViewBoundsAsync(box, null, MapAnimationKind.Default);

            // Collapse the progress bar.
            pbProgressBar.Visibility = Windows.UI.Xaml.Visibility.Collapsed;
        }

        protected override void OnNavigatedFrom(NavigationEventArgs e)
        {
            this.navigationHelper.OnNavigatedFrom(e);
        }

        #endregion
    }
}