﻿using MapControlSamples.Common;
using System;
using Windows.ApplicationModel.Resources;
using Windows.Networking.Connectivity;
using Windows.Services.Maps;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Navigation;

namespace MapControlSamples
{
    public sealed partial class MenuPage : Page
    {
        private readonly NavigationHelper navigationHelper;
        private readonly ObservableDictionary defaultViewModel = new ObservableDictionary();
        private readonly ResourceLoader resourceLoader = ResourceLoader.GetForCurrentView("Resources");

        public MenuPage()
        {
            this.InitializeComponent();

            this.NavigationCacheMode = NavigationCacheMode.Required;

            this.navigationHelper = new NavigationHelper(this);
            this.navigationHelper.LoadState += this.NavigationHelper_LoadState;
            this.navigationHelper.SaveState += this.NavigationHelper_SaveState;
        }

        /// <summary>
        /// Gets the <see cref="NavigationHelper"/> associated with this <see cref="Page"/>.
        /// </summary>
        public NavigationHelper NavigationHelper
        {
            get { return this.navigationHelper; }
        }

        /// <summary>
        /// Gets the view model for this <see cref="Page"/>.
        /// This can be changed to a strongly typed view model.
        /// </summary>
        public ObservableDictionary DefaultViewModel
        {
            get { return this.defaultViewModel; }
        }

        /// <summary>
        /// Populates the page with content passed during navigation. Any saved state is also
        /// provided when recreating a page from a prior session.
        /// </summary>
        /// <param name="sender">
        /// The source of the event; typically <see cref="NavigationHelper"/>.
        /// </param>
        /// <param name="e">Event data that provides both the navigation parameter passed to
        /// <see cref="Frame.Navigate(Type, Object)"/> when this page was initially requested and
        /// a dictionary of state preserved by this page during an earlier
        /// session. The state will be null the first time a page is visited.</param>
        private void NavigationHelper_LoadState(object sender, LoadStateEventArgs e)
        {
        }

        /// <summary>
        /// Preserves state associated with this page in case the application is suspended or the
        /// page is discarded from the navigation cache. Values must conform to the serialization
        /// requirements of <see cref="SuspensionManager.SessionState"/>.
        /// </summary>
        /// <param name="sender">The source of the event; typically <see cref="NavigationHelper"/>.</param>
        /// <param name="e">Event data that provides an empty dictionary to be populated with
        /// serializable state.</param>
        private void NavigationHelper_SaveState(object sender, SaveStateEventArgs e)
        {
            // TODO: Save the unique state of the page here.
        }

        /// <summary>
        /// Adds an item to the list when the app bar button is clicked.
        /// </summary>
        private void AddAppBarButton_Click(object sender, RoutedEventArgs e)
        {
        }


        #region NavigationHelper registration

        /// <summary>
        /// The methods provided in this section are simply used to allow
        /// NavigationHelper to respond to the page's navigation methods.
        /// <para>
        /// Page specific logic should be placed in event handlers for the  
        /// <see cref="NavigationHelper.LoadState"/>
        /// and <see cref="NavigationHelper.SaveState"/>.
        /// The navigation parameter is available in the LoadState method 
        /// in addition to page state preserved during an earlier session.
        /// </para>
        /// </summary>
        /// <param name="e">Provides data for navigation methods and event
        /// handlers that cannot cancel the navigation request.</param>
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            this.navigationHelper.OnNavigatedTo(e);
        }

        protected override void OnNavigatedFrom(NavigationEventArgs e)
        {
            this.navigationHelper.OnNavigatedFrom(e);
        }

        #endregion

        private void btnBasicMap_Tapped(object sender, TappedRoutedEventArgs e)
        {
            if (!Frame.Navigate(typeof(BasicMapPage)))
            {
                throw new Exception(this.resourceLoader.GetString("NavigationFailedExceptionMessage"));
            }
        }

        private void btnDefaultImage_Tapped(object sender, TappedRoutedEventArgs e)
        {
            if (!Frame.Navigate(typeof(DefaultImagePage)))
            {
                throw new Exception(this.resourceLoader.GetString("NavigationFailedExceptionMessage"));
            }
        }

        private void btnCustomImage_Tapped(object sender, TappedRoutedEventArgs e)
        {
            if (!Frame.Navigate(typeof(CustomImagePage)))
            {
                throw new Exception(this.resourceLoader.GetString("NavigationFailedExceptionMessage"));
            }
        }

        private void btnGeocoding_Tapped(object sender, TappedRoutedEventArgs e)
        {
            if (!Frame.Navigate(typeof(GeocodingPage)))
            {
                throw new Exception(this.resourceLoader.GetString("NavigationFailedExceptionMessage"));
            }
        }

        private void btnReverseGeocoding_Tapped(object sender, TappedRoutedEventArgs e)
        {
            if (!Frame.Navigate(typeof(ReverseGeocodingPage)))
            {
                throw new Exception(this.resourceLoader.GetString("NavigationFailedExceptionMessage"));
            }
        }

        private void btnControlsCode_Tapped(object sender, TappedRoutedEventArgs e)
        {
            if (!Frame.Navigate(typeof(ControlsCodePage)))
            {
                throw new Exception(this.resourceLoader.GetString("NavigationFailedExceptionMessage"));
            }
        }

        private void btnDrivingRoutes_Tapped(object sender, TappedRoutedEventArgs e)
        {
            if (!Frame.Navigate(typeof(DrivingRoutesPage)))
            {
                throw new Exception(this.resourceLoader.GetString("NavigationFailedExceptionMessage"));
            }
        }

        private void btnWalkingRoutes_Tapped(object sender, TappedRoutedEventArgs e)
        {
            if (!Frame.Navigate(typeof(WalkingRoutesPage)))
            {
                throw new Exception(this.resourceLoader.GetString("NavigationFailedExceptionMessage"));
            }
        }

        private async void btnDownloadingMaps_Tapped(object sender, TappedRoutedEventArgs e)
        {
            // Determine if there is internet connectivity.
            ConnectionProfile profile = NetworkInformation.GetInternetConnectionProfile();

            bool internet = false;
            if (profile == null) internet = false;
            else if (profile.GetNetworkConnectivityLevel() == NetworkConnectivityLevel.InternetAccess) internet = true;

            // Message dialog telling the user there is no internet connection.
            if (!internet)
            {
                // Tell the user they are on a metered connection.
                string noInternetMessage = this.resourceLoader.GetString("NoInternetMessage");
                string noInternetTitle = this.resourceLoader.GetString("NoInternetTitle");
                MessageDialog noInternetMessageDialog = new MessageDialog(noInternetMessage, noInternetTitle);
                await noInternetMessageDialog.ShowAsync();

                return;
            }

            // Get the cost associated with the connection.
            ConnectionCost cost = profile.GetConnectionCost();

            // If there is a cost then show a message dialog and give the user the option to cancel.
            if (cost.NetworkCostType != NetworkCostType.Unrestricted)
            {
                // Tell the user they are on a metered connection.
                string message = this.resourceLoader.GetString("ConnectionWarning");
                string title = this.resourceLoader.GetString("ConnectionWarningTitle");
                MessageDialog messageDialog = new MessageDialog(message, title);

                // Add commands and set their callbacks; both buttons use the same callback function instead of inline event handlers
                messageDialog.Commands.Add(new UICommand(
                    "OK",
                    new UICommandInvokedHandler(this.DownloadHandler)));
                
                messageDialog.Commands.Add(new UICommand(
                    "Cancel",
                    new UICommandInvokedHandler(this.DownloadHandler)));

                // Set the command that will be invoked by default
                messageDialog.DefaultCommandIndex = 1;

                // Set the command to be invoked when escape is pressed
                messageDialog.CancelCommandIndex = 1;

                await messageDialog.ShowAsync();
            }
            else
            {
                MapManager.ShowDownloadedMapsUI();
            }

        }

        private void DownloadHandler(IUICommand command)
        {
            if (command.Label == "OK") MapManager.ShowDownloadedMapsUI();
        }

        private async void btnUpdatingMaps_Tapped(object sender, TappedRoutedEventArgs e)
        {
            // Determine if there is internet connectivity.
            ConnectionProfile profile = NetworkInformation.GetInternetConnectionProfile();

            bool internet = false;
            if (profile == null) internet = false;
            else if (profile.GetNetworkConnectivityLevel() == NetworkConnectivityLevel.InternetAccess) internet = true;

            // Message dialog telling the user there is no internet connection.
            if (!internet)
            {
                // Tell the user they are on a metered connection.
                string noInternetMessage = this.resourceLoader.GetString("NoInternetMessage");
                string noInternetTitle = this.resourceLoader.GetString("NoInternetTitle");
                MessageDialog noInternetMessageDialog = new MessageDialog(noInternetMessage, noInternetTitle);
                await noInternetMessageDialog.ShowAsync();

                return;
            }

            // Get the cost associated with the connection.
            ConnectionCost cost = profile.GetConnectionCost();

            // If there is a cost then show a message dialog and give the user the option to cancel.
            if (cost.NetworkCostType != NetworkCostType.Unrestricted)
            {
                // Tell the user they are on a metered connection.
                string message = this.resourceLoader.GetString("ConnectionWarning");
                string title = this.resourceLoader.GetString("ConnectionWarningTitle");
                MessageDialog messageDialog = new MessageDialog(message, title);

                // Add commands and set their callbacks; both buttons use the same callback function instead of inline event handlers
                messageDialog.Commands.Add(new UICommand(
                    "OK",
                    new UICommandInvokedHandler(this.DownloadHandler)));

                messageDialog.Commands.Add(new UICommand(
                    "Cancel",
                    new UICommandInvokedHandler(this.DownloadHandler)));

                // Set the command that will be invoked by default
                messageDialog.DefaultCommandIndex = 1;

                // Set the command to be invoked when escape is pressed
                messageDialog.CancelCommandIndex = 1;

                await messageDialog.ShowAsync();
            }
            else
            {
                MapManager.ShowMapsUpdateUI();
            }

        }

        private void UpdateHandler(IUICommand command)
        {
            if (command.Label == "OK") MapManager.ShowMapsUpdateUI();
        }

        private void btnControlsXaml_Tapped(object sender, TappedRoutedEventArgs e)
        {
            if (!Frame.Navigate(typeof(ControlsXamlPage)))
            {
                throw new Exception(this.resourceLoader.GetString("NavigationFailedExceptionMessage"));
            }
        }

        private void btnCollection_Tapped(object sender, TappedRoutedEventArgs e)
        {
            if (!Frame.Navigate(typeof(BindingCollectionsPage)))
            {
                throw new Exception(this.resourceLoader.GetString("NavigationFailedExceptionMessage"));
            }

        }

        private void btnScale_Tapped(object sender, TappedRoutedEventArgs e)
        {
            if (!Frame.Navigate(typeof(ScaleControlsPage)))
            {
                throw new Exception(this.resourceLoader.GetString("NavigationFailedExceptionMessage"));
            }

        }

        private void btnAccuracyCache_Tapped(object sender, TappedRoutedEventArgs e)
        {
            if (!Frame.Navigate(typeof(AccuracyCachePage)))
            {
                throw new Exception(this.resourceLoader.GetString("NavigationFailedExceptionMessage"));
            }

        }

    }
}
