﻿using MapControlSamples.Common;
using System;
using Windows.ApplicationModel.Resources;
using Windows.Devices.Geolocation;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Maps;
using Windows.UI.Xaml.Navigation;
using MapControlSamples.Models;

namespace MapControlSamples
{

    /// <summary>
    /// A page that displays details for a single item within a group.
    /// </summary>
    public sealed partial class ScaleControlsPage : Page
    {
        private readonly NavigationHelper navigationHelper;
        private readonly ResourceLoader resourceLoader = ResourceLoader.GetForCurrentView("Resources");

        public ScaleControlsPage()
        {
            this.InitializeComponent();

            this.navigationHelper = new NavigationHelper(this);
            this.navigationHelper.LoadState += this.NavigationHelper_LoadState;
            this.navigationHelper.SaveState += this.NavigationHelper_SaveState;

        } 

        /// <summary>
        /// Gets the <see cref="NavigationHelper"/> associated with this <see cref="Page"/>.
        /// </summary>
        public NavigationHelper NavigationHelper
        {
            get { return this.navigationHelper; }
        }

        /// <summary>
        /// Populates the page with content passed during navigation. Any saved state is also
        /// provided when recreating a page from a prior session.
        /// </summary>
        /// <param name="sender">
        /// The source of the event; typically <see cref="NavigationHelper"/>.
        /// </param>
        /// <param name="e">Event data that provides both the navigation parameter passed to
        /// <see cref="Frame.Navigate(Type, Object)"/> when this page was initially requested and
        /// a dictionary of state preserved by this page during an earlier
        /// session.  The state will be null the first time a page is visited.</param>
        private void NavigationHelper_LoadState(object sender, LoadStateEventArgs e)
        {
            // TODO: Create an appropriate data model for your problem domain to replace the sample data.
        }

        /// <summary>
        /// Preserves state associated with this page in case the application is suspended or the
        /// page is discarded from the navigation cache.  Values must conform to the serialization
        /// requirements of <see cref="SuspensionManager.SessionState"/>.
        /// </summary>
        /// <param name="sender">The source of the event; typically <see cref="NavigationHelper"/>.</param>
        /// <param name="e">Event data that provides an empty dictionary to be populated with
        /// serializable state.</param>
        private void NavigationHelper_SaveState(object sender, SaveStateEventArgs e)
        {
            // TODO: Save the unique state of the page here.
        }

        #region NavigationHelper registration

        /// <summary>
        /// The methods provided in this section are simply used to allow
        /// NavigationHelper to respond to the page's navigation methods.
        /// <para>
        /// Page specific logic should be placed in event handlers for the  
        /// <see cref="NavigationHelper.LoadState"/>
        /// and <see cref="NavigationHelper.SaveState"/>.
        /// The navigation parameter is available in the LoadState method 
        /// in addition to page state preserved during an earlier session.
        /// </para>
        /// </summary>
        /// <param name="e">Provides data for navigation methods and event
        /// handlers that cannot cancel the navigation request.</param>
        protected async override void OnNavigatedTo(NavigationEventArgs e)
        {
            // Call the navigation helper base function.
            this.navigationHelper.OnNavigatedTo(e);

            // Size the Map Control
            myMapControl.Width = Windows.UI.Xaml.Window.Current.Bounds.Width;
            myMapControl.Height = Windows.UI.Xaml.Window.Current.Bounds.Height - 85;

            // Get the user's current location so that it can be used as the center
            // point of the map control.
            Geolocator geolocator = new Geolocator();
            Geoposition geoposition = null;
            try
            {
                geoposition = await geolocator.GetGeopositionAsync();
            }
            catch (Exception)
            {
                // Make sure you have defined ID_CAP_LOCATION in the application manifest 
                // and that on your phone, you have turned on location by checking 
                // Settings > Location.

                // Handle errors like unauthorized access to location services
            }

            // Set the map center point.
            LocationEntity locationEntity = new LocationEntity();
            locationEntity.Location = geoposition.Coordinate.Point;
            locationEntity.Message = "You are here";

            // Specify the location that will be at the center of the map.
            myMapControl.Center = locationEntity.Location;

            // Set the map controls data context so data binding will work.
            myMapControl.DataContext = locationEntity;

            // Zoom level.  
            myMapControl.ZoomLevel = 15;
            
            // The map control has done most of its work.  Make the controls visible
            // and collapse the progress bar.
            imgMyLocation.Visibility = Windows.UI.Xaml.Visibility.Visible;
            borderMessage.Visibility = Windows.UI.Xaml.Visibility.Visible;
            pbProgressBar.Visibility = Windows.UI.Xaml.Visibility.Collapsed;
        }

        protected override void OnNavigatedFrom(NavigationEventArgs e)
        {
            this.navigationHelper.OnNavigatedFrom(e);
        }

        #endregion

        private void myMapControl_ZoomLevelChanged(MapControl sender, object args)
        {
            // Get the map control's current zoom level.
            double zoomLevel = sender.ZoomLevel;

            // Use the latitude from the center point of the map control.
            double latitude = sender.Center.Position.Latitude;

            // Hard coded test values so that the results of the equation below can be compared to the chart at
            // http://msdn.microsoft.com/en-us/library/aa940990.aspx.
            //latitude = 0;
            //zoomLevel = 1;

            // The following formula for map resolution  needs latitude in radians.
            latitude = latitude * (Math.PI / 180);

            // This constant is based on the diameter of the Earth and the equations Microsoft uses to determine the map 
            // shown for the map control's zoom level.
            const double BING_MAP_CONSTANT = 156543.04;

            // Calculate the number of meters represented by a single pixel. 
            double MetersPerPixel = BING_MAP_CONSTANT * Math.Cos(latitude) / Math.Pow(2, zoomLevel);
            double KilometersPerPixel = MetersPerPixel / 1000;
            double FeetPerPixel = MetersPerPixel * 3.28;
            double MilesPerPixel = FeetPerPixel / 5280;

            if (System.Globalization.RegionInfo.CurrentRegion.IsMetric)
            {
                // Determine the distance represented by the rectangle control.
                double scaleDistance = recScale.Width * KilometersPerPixel;

                // Use meters if scale distance is less than 1 kilometer
                if (scaleDistance < 1)
                {
                    scaleDistance = recScale.Width * MetersPerPixel;
                    scaleDistance = Math.Round(scaleDistance, 1);
                    tbScale.Text = scaleDistance.ToString() + " m";
                }
                else
                {
                    scaleDistance = Math.Round(scaleDistance, 1);
                    tbScale.Text = scaleDistance.ToString() + " km";
                }
            }
            else
            {
                // Determine the distance represented by the rectangle control.
                double scaleDistance = recScale.Width * MilesPerPixel;

                // Use meters if scale distance is less than 1 kilometer
                if (scaleDistance < 1)
                {
                    scaleDistance = recScale.Width * FeetPerPixel;
                    scaleDistance = Math.Round(scaleDistance, 1);
                    tbScale.Text = scaleDistance.ToString() + " feet";
                }
                else
                {
                    scaleDistance = Math.Round(scaleDistance, 1);
                    tbScale.Text = scaleDistance.ToString() + " miles";
                }

            }

        }
    
    
    
    }
}